@@ -162,8 +162,28 @@ proc xcircuit::setpstype {mode} {
    xcircuit::page encapsulation $mode
 }
 
+proc xcircuit::dolinks {} {
+   global imultiple dmultiple
+   set ilinks [xcircuit::page links independent]
+   if {$ilinks > 1} {
+      set imultiple 1
+   } else {
+      set imultiple 0
+   }
+   if {$ilinks == 1} { set plural ""} else { set plural "s"}
+   .output.title.imulti configure -text "$ilinks schematic$plural"
+
+   if {$dmultiple == 1} {
+      set dlinks [xcircuit::page links dependent]
+   } else {
+      set dlinks 0
+   }
+   if {$dlinks == 1} { set plural ""} else { set plural "s"}
+   .output.title.dmulti configure -text "$dlinks subcircuit$plural"
+}
+
 proc xcircuit::pageupdate { {subcommand "none"} } {
-   global tcl_precision autofit orient multiple
+   global tcl_precision autofit orient imultiple dmultiple
    if {[info level] <= 1} {
      switch -- $subcommand {
 	save {
@@ -210,15 +230,8 @@ proc xcircuit::pageupdate { {subcommand "none"} } {
 	  } else {
 	     .output.textent.buto configure -text Landscape
 	  }
-	  ::set links [xcircuit::page links]
-	  if {$links > 1} {
-	    set multiple 1
-	    .output.title.multi configure -text "$links pages"
-	    pack .output.title.multi -side right
-	  } else {
-	    set multiple 0
-	    pack forget .output.title.multi
-	  }
+	  xcircuit::dolinks
+
 	  ::set autofit [xcircuit::page fit]
 	  if {[string match *.* $fname] == 0} {append fname .ps}
 	  if {[glob -nocomplain ${fname}] == {}} {
@@ -228,7 +241,8 @@ proc xcircuit::pageupdate { {subcommand "none"} } {
 	  }
 	  .output.bbar.okay configure -command \
 		{xcircuit::page filename [.output.textent.txt1 get]; \
-		 xcircuit::page save; wm withdraw .output}
+		 if {$dmultiple == 1} {xcircuit::page save} else { \
+		 xcircuit::page saveonly }; wm withdraw .output}
         }
      }
   }
@@ -340,6 +354,18 @@ proc xcircuit::coordstyle { { mode get } } {
    }
 }
 
+proc xcircuit::raiseconsole {} {
+   xcircuit::consoleup
+   .xcircuit.menubar.filebutton.filemenu entryconfigure \
+	 4 -label "No Console" -command {xcircuit::lowerconsole}
+}
+
+proc xcircuit::lowerconsole {} {
+   xcircuit::consoledown
+   .xcircuit.menubar.filebutton.filemenu entryconfigure \
+	 4 -label "Tcl Console" -command {xcircuit::raiseconsole}
+}
+
 #----------------------------------------------------------------------
 # Command tags---these let the command-line entry functions update the
 # Tk windows, so that the Tk window structure doesn't need to be hard-
@@ -422,7 +448,8 @@ bind .filelist.textent.txt <Return> \
 
 # First, set the variables associated with toggle and radio buttons
 set autofit 0
-set multiple 1
+set imultiple 0
+set dmultiple 1
 
 toplevel .output -bg beige
 wm withdraw .output
@@ -436,9 +463,16 @@ pack .output.textent -side top -padx 20 -pady 7 -fill x
 pack .output.bbar -side top -padx 20 -pady 7 -fill x
 
 label .output.title.field -text "PostScript output properties (Page 1):" -bg tan
-checkbutton .output.title.multi -text "1 page" -bg beige -variable multiple \
-	-command {.output.textent.txt1 delete 0 end; .output.textent.but1 configure \
-	-text Apply; xcircuit::page filename {}; focus .output.textent.txt1}
+checkbutton .output.title.imulti -text "1 schematic" -bg beige -variable imultiple \
+	-command {xcircuit::dolinks ; if {$imultiple == 1} {.output.textent.txt1 \
+	delete 0 end; .output.textent.but1 configure -text Apply; xcircuit::page \
+	filename {}; focus .output.textent.txt1 ; xcircuit::dolinks }}
+checkbutton .output.title.dmulti -text "0 subcircuits" -bg beige -variable dmultiple \
+	-command {xcircuit::dolinks ; .output.textent.txt1 \
+	delete 0 end; .output.textent.but1 configure -text Apply; xcircuit::page \
+	filename {}; focus .output.textent.txt1 ; xcircuit::dolinks }
+pack .output.title.dmulti -side right -padx 5
+pack .output.title.imulti -side right -padx 5
 
 pack .output.title.field -side left
 
@@ -463,7 +497,7 @@ menubutton .output.textent.butp -text "Embedded (EPS)" -bg beige \
 
 checkbutton .output.textent.butf -text "Auto-fit" -bg beige -variable autofit \
 	-onvalue true -offvalue false -command {xcircuit::page fit $autofit}
-frame .output.textent.txtf
+frame .output.textent.txtf -bg beige
 menubutton .output.textent.txtf.sizb -text "Sizes" -bg beige \
 	-menu .output.textent.txtf.sizb.sizemenu
 entry .output.textent.txtf.txtp -bg white -relief sunken -width 14
@@ -545,13 +579,13 @@ pack .output.bbar.okay -side left -ipadx 10
 pack .output.bbar.cancel -side right -ipadx 10
 
 set m [menu .output.textent.txtf.sizb.sizemenu -tearoff 0]
-$m add radio -label "Letter" -variable sheetsize -value letter -command \
+$m add radio -label "Letter (ANSI A)" -variable sheetsize -value letter -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "8.5 x 11.0 in"}
 $m add radio -label "Legal" -variable sheetsize -value legal -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "8.5 x 14.0 in"}
 $m add radio -label "Statement" -variable sheetsize -value statement -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "5.5 x 8.5 in"}
-$m add radio -label "Tabloid" -variable sheetsize -value tabloid -command \
+$m add radio -label "Tabloid (ANSI B)" -variable sheetsize -value tabloid -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "11.0 x 17.0 in"}
 $m add radio -label "Ledger" -variable sheetsize -value ledger -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "17.0 x 11.0 in"}
@@ -563,15 +597,21 @@ $m add radio -label "10x14" -variable sheetsize -value tenfourteen -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "10.0 x 14.0 in"}
 $m add radio -label "Executive" -variable sheetsize -value executive -command \
 	{ xcircuit::coordstyle inches; xcircuit::page size "7.5 x 10.0 in"}
+$m add radio -label "ANSI C" -variable sheetsize -value ansic -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "17.0 x 22.0 in"}
+$m add radio -label "ANSI D" -variable sheetsize -value ansid -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "22.0 x 34.0 in"}
+$m add radio -label "ANSI E" -variable sheetsize -value ansie -command \
+	{ xcircuit::coordstyle inches; xcircuit::page size "34.0 x 44.0 in"}
 $m add radio -label "A3" -variable sheetsize -value a3 -command \
-	{ xcircuit::coordstyle centimeters; xcircuit::page size "30 x 42 cm"}
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "29.7 x 42.0 cm"}
 $m add radio -label "A4" -variable sheetsize -value a4 -command \
-	{ xcircuit::coordstyle centimeters; xcircuit::page size "21 x 30 cm"}
+	{ xcircuit::coordstyle centimeters; xcircuit::page size "21.0 x 29.7 cm"}
 $m add radio -label "A5" -variable sheetsize -value a5 -command \
 	{ xcircuit::coordstyle centimeters; xcircuit::page size "14.82 x 18.43 cm"}
-$m add radio -label "B4" -variable sheetsize -value a5 -command \
+$m add radio -label "B4" -variable sheetsize -value b4 -command \
 	{ xcircuit::coordstyle centimeters; xcircuit::page size "25.7 x 36.4 cm"}
-$m add radio -label "B5" -variable sheetsize -value a5 -command \
+$m add radio -label "B5" -variable sheetsize -value b5 -command \
 	{ xcircuit::coordstyle centimeters; xcircuit::page size "18.2 x 25.7 cm"}
 
 #--------------------------------------
@@ -1187,7 +1227,7 @@ $m add command -label "Read XCircuit File" -command {xcircuit::promptloadfile}
 $m add command -label "Import XCircuit PS" -command {xcircuit::promptimportfile}
 $m add command -label "Import background PS" -command {xcircuit::promptimportbackground}
 $m add command -label "Execute Script" -command {xcircuit::promptexecscript}
-$m add command -label "Tcl Console" -command {xcircuit::consoleup}
+$m add command -label "Tcl Console" -command {xcircuit::raiseconsole}
 $m add command -label "Write XCircuit PS" -command {xcircuit::promptsavepage}
 $m add separator
 $m add command -label "Add To Library" -command {xcircuit::promptaddlibrary}
@@ -1567,10 +1607,43 @@ xcircuit::newpagebutton "Page 10" 10
 
 xcircuit::newlibrarybutton "User Library"
 
-bind .xcircuit.mainframe.mainarea.drawing <Enter> {focus %W}
+# New key bindings should pass through this function so that key
+# strokes are captured correctly for label editing.
 
+proc xcircuit::keyaction {keycode {keystate 0}} {
+   switch -- [eventmode] {
+      text2 -
+      text3 -
+      cattext {
+	 standardaction $keycode key $keystate
+	 return true
+      }
+   }
+   return false
+}
+
+# Wrapper procedure to (re)bind a key to a Tcl procedure (Ed Casas 9/4/03)
+
+proc xcircuit::keybind { key proc } { bind .xcircuit.mainframe.mainarea.drawing \
+	$key "if \{!\[xcircuit::keyaction %k %s\]\} \{ $proc \}" }
+
+# Add key and button bindings for XCircuit commands (standard actions)
+# These can be overridden by binding to specific keys and/or buttons.
+
+bind .xcircuit.mainframe.mainarea.drawing <ButtonPress> {standardaction %b down %s}
+bind .xcircuit.mainframe.mainarea.drawing <ButtonRelease> {standardaction %b up %s}
+bind .xcircuit.mainframe.mainarea.drawing <KeyPress> {standardaction %k key %s}
+
+# Here are some extra key functions that come with the TCL wrapper
+
+bind .xcircuit.mainframe.mainarea.drawing <Key-Next> {catch {page [expr {[page] + 1}]}}
+bind .xcircuit.mainframe.mainarea.drawing <Key-Prior> {catch {page [expr {[page] - 1}]}}
+
+# Wait for the drawing area to become visible, and set the focus on it.
+
+bind .xcircuit.mainframe.mainarea.drawing <Enter> {focus %W}
 tkwait visibility .xcircuit.mainframe.mainarea.drawing
-focus .xcircuit.mainframe.mainarea.drawing
+focus -force .xcircuit.mainframe.mainarea.drawing
 
 # This pops down the interpreter window, if the "console.tcl" script was run.
 
