@@ -120,6 +120,37 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
    if (mode) drawarea(NULL, NULL, NULL);
 }
 
+/*----------------------------------------------------------------------*/
+/* Find the object with the indicated name.				*/
+/*----------------------------------------------------------------------*/
+
+objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
+{
+   int i;
+   liblistptr spec;
+
+   for (i = 0; i < xobjs.numlibs; i++) {
+      for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
+         if (!strcmp(objname, spec->thisinst->thisobject->name)) {
+	    *ret_inst = spec->thisinst;
+	    return spec->thisinst->thisobject;
+	 }
+      }
+   }
+
+   if (dopages) {
+      for (i = 0; i < xobjs.pages; i++) {
+	 if (xobjs.pagelist[i]->pageinst == NULL) continue;
+	 if (!strcmp(objname, xobjs.pagelist[i]->pageinst->thisobject->name)) {
+	    *ret_inst = xobjs.pagelist[i]->pageinst;
+	    return xobjs.pagelist[i]->pageinst->thisobject;
+	 }
+      }
+   }
+   
+   return NULL;
+}
+
 /*------------------------------------------------------------------------*/
 /* Ensure that a page name is unique (required for schematic association) */
 /*------------------------------------------------------------------------*/
@@ -859,7 +890,8 @@ void swapschem(xcWidget w, pointertype mode, caddr_t calldata)
       /* If symschem matches the last entry on the push stack, then we	*/
       /* pop; otherwise, we push.					*/
 
-      if (areastruct.stack && areastruct.stack->thisinst->thisobject == topobject->symschem) {
+      if (areastruct.stack && areastruct.stack->thisinst->thisobject
+			== topobject->symschem) {
 	 areastruct.topinstance = areastruct.stack->thisinst;
 	 pop_stack(&areastruct.stack);
       }
